/*  Gtk+ Widget Factory
 *  Copyright (C) 1999-2000  Jeroen Benckhuijsen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "gwf_plugin.h"
#include <gtk/gtk.h>
#include <gmodule.h>
#include "gwf_project.h"
#include "gwf_xmlio.h"
#include "gwf_settings.h"
#include "gwf_projlist.h"
#include "gwf_error.h"
#include "gwf_tools.h"
#include <string.h>

typedef struct _GwfPlugForeach		GwfPlugForeach;

struct _GwfPlugForeach
{
  GwfPlugForeachFunc	func;
  gpointer 		user_data;
};


static gboolean		gwf_pluginlist_remove_foreach		(gpointer key,gpointer value,gpointer user_data);
static void		foreach_plugin 				(gpointer key,gpointer value,gpointer user_data);

static void		gwf_pluginlist_reg_errors_foreach	(gpointer key ,gpointer value, gpointer user_data);

static void		gwf_pluginlist_reg_settings_foreach 	(gpointer key,gpointer value,gpointer user_data);
static GwfSettings*	gwf_dataplug_create_settings		(GwfPlugin *plugin);

static void		gwf_pluginlist_reg_xmlio_foreach	(gpointer key,gpointer value,gpointer user_data);
static gint		gwf_dataplug_create_xmlio		(GwfPlugin *plugin,GwfXmlIO *xml_io);

static void		gwf_pluginlist_reg_data_foreach		(gpointer key,gpointer value,gpointer user_data);
static void		gwf_pluginlist_reg_project_data		(GwfProjlist *projlist,gchar *name,gpointer user_data);
static GwfProjData*	gwf_dataplug_create_projdata		(GwfPlugin *plugin,GwfProject *project);

static void		gwf_pluginlist_reg_treedatas_foreach	(gpointer key,gpointer value,gpointer user_data);
static GwfTreeData*	gwf_pluginlist_create_treedata		(GwfPlugin *plugin);

static void		gwf_pluginlist_reg_source_foreach	(gpointer key,gpointer value,gpointer user_data);



GwfPluginList*
gwf_pluginlist_new ()
{
  GwfPluginList	*tmp;
   
  tmp = g_malloc(sizeof(GwfPluginList));
   
  tmp->datas   = g_hash_table_new(g_str_hash,g_str_equal);

  return tmp; 
}

void
gwf_pluginlist_free (GwfPluginList *list)
{
  g_return_if_fail(list != NULL);
  
  g_hash_table_foreach_remove(list->datas,gwf_pluginlist_remove_foreach,NULL);
  g_hash_table_destroy(list->datas);
}

static gboolean
gwf_pluginlist_remove_foreach(gpointer key,gpointer value,gpointer user_data)
{
  gwf_plugin_unload((GwfPlugin*)value);
  return TRUE;
}

gint
gwf_pluginlist_add_plugin (GwfPluginList *list,GwfPlugin *plugin)
{
  g_return_val_if_fail(list != NULL,0);
  g_return_val_if_fail(plugin != NULL,0);
  
  if (g_hash_table_lookup(list->datas,plugin->name))
    return 0;
    
  g_hash_table_insert(list->datas,(gpointer)plugin->name,(gpointer)plugin);
  return 1;
}

void
gwf_pluginlist_del_plugin (GwfPluginList *list,gchar *name)
{
  g_return_if_fail(list != NULL);
  g_return_if_fail(name != NULL);

  g_hash_table_remove(list->datas,(gconstpointer)name);
}

GwfPlugin*
gwf_pluginlist_get_plugin (GwfPluginList *list,gchar *name)
{
  g_return_val_if_fail(list != NULL,NULL);
  g_return_val_if_fail(name != NULL,NULL);

  return (GwfPlugin*)g_hash_table_lookup(list->datas,(gconstpointer)name);
}

void
gwf_pluginlist_foreach_plugin (GwfPluginList *list,GwfPlugForeachFunc func,gpointer user_data)
{
  GwfPlugForeach	tmp;

  tmp.func = (GwfPlugForeachFunc)func;
  tmp.user_data = user_data;
  g_hash_table_foreach(list->datas,foreach_plugin,(gpointer)&tmp);
}

static void
foreach_plugin (gpointer key,gpointer value,gpointer user_data)
{
  GwfPlugForeach	*tmp = user_data;

  tmp->func((GwfPlugin*)value,tmp->user_data);
}

GwfPlugin*
gwf_plugin_new (gchar *name)
{
  GwfPlugin	*plugin;

  g_return_val_if_fail(name != NULL,NULL);

  plugin = g_malloc0(sizeof(GwfPlugin));

  plugin->name = g_strdup(name);
  plugin->errmsg = g_ptr_array_new();

  return plugin;
}

void
gwf_plugin_free (GwfPlugin *plugin)
{
  GList		*tmp;

  g_return_if_fail(plugin != NULL);

  g_free(plugin->name);
  g_free(plugin->href);
  g_free(plugin->version);

  for (tmp = plugin->source_write ; tmp ; tmp = tmp->next)
    g_free(tmp->data);
  g_list_free(plugin->source_write);

  g_free(plugin);
}

void
gwf_plugin_add_srcwrt (GwfPlugin *plugin,GwfWritePrior prior,GwfSrcWrtFunc func,GwfWriteFile file)
{
  GwfSrcPlugData	*tmp;

  g_return_if_fail(plugin != NULL);

  tmp = g_malloc(sizeof(GwfSrcPlugData));
  tmp->prior = prior;
  tmp->func = func;
  tmp->file = file;

  plugin->source_write = g_list_prepend(plugin->source_write,(gpointer)tmp);
}

/*************************/
/* Plugin load functions */
/*************************/

void
gwf_pluginlist_load_dir (GwfPluginList *list,gchar *dir,GwfError *error)
{
  GwfPlugin		*plugin;
  gchar			*filename;
  gchar			*tmp;
  GList			*fnames;
  gchar			*ext;

  g_return_if_fail(list != NULL);
  g_return_if_fail(dir != NULL);

  fnames = gwf_open_dir(dir);
  if (!fnames)
  {
    gwf_error_raise_error(error,"Plugins : No plugin dir");
    return;
  }

  tmp = g_module_build_path("","");
  ext = g_strdup(strrchr(tmp,'.'));
  g_free(tmp);

  for ( ; fnames ; fnames = g_list_remove(fnames,fnames->data) )
  {
    tmp = (gchar*)fnames->data;
    if ((strcmp(tmp,".") != 0) && (strcmp(tmp,"..") != 0) && (strcmp(tmp + strlen(tmp) - strlen(ext),ext) == 0) )
    {
      filename = g_strdup_printf("%s/%s",dir,tmp);
      plugin = gwf_plugin_load(filename);
      g_free(filename);

      if (plugin)
      {
        if (!gwf_pluginlist_add_plugin(list,plugin))
          gwf_error_raise_error(error,"Plugins : Duplicate plugin");
      }
      else
      {
        gwf_error_raise_error(error,"Plugins : Invalid plugin");
      }
    }
    g_free(fnames->data);
  }

  g_free(ext);
}

GwfPlugin*
gwf_plugin_load	(gchar *filename)
{
  GModule		*module;
  GwfPluginNewFunc	newfunc;
  GwfPlugin		*plugin;

  module = g_module_open(filename,0);
  if (!module)
    return NULL;

  if (!g_module_symbol(module,"gwf_new_plugin",(gpointer*)&newfunc))
  {
    g_module_close(module);
    return NULL;
  }

  plugin = newfunc();
  if (!plugin)
    return NULL;

  plugin->module = module;

  if (!plugin->name || !plugin->version || (strcmp(plugin->name,"Basic") == 0) ||
      !plugin->load_plugin || !plugin->save_plugin || !plugin->new_data ||
      !plugin->setting_create || !plugin->source_check || !plugin->source_write || !plugin->tree_attach)
  {
    gwf_plugin_free(plugin);
    return NULL;
  }

  return plugin;
}


/***************************/
/* Plugin unload functions */
/***************************/

gint
gwf_plugin_unload (GwfPlugin *plugin)
{
  GModule		*module;

  g_return_val_if_fail(plugin != NULL,0);
  
  module = plugin->module;
  gwf_plugin_free(plugin);

  return (module) ? g_module_close(module) : 1;
}


/***********************/
/* Plugin registration */
/***********************/

/******************************/
/* Error message registration */
/******************************/

void
gwf_pluginlist_reg_errors(GwfPluginList *list,GwfError *error)
{
  g_return_if_fail(list != NULL);
  g_return_if_fail(error != NULL);

  g_hash_table_foreach(list->datas,gwf_pluginlist_reg_errors_foreach,(gpointer)error);
}

void
gwf_plugin_reg_errors(GwfPlugin *plugin,GwfError *error)
{
  GwfErrData	*errdat;
  gint		i;

  g_return_if_fail(plugin != NULL);
  g_return_if_fail(error != NULL);

  for (i = 0 ; i < plugin->errmsg->len ; i++)
  {
    errdat = (GwfErrData*)g_ptr_array_index(plugin->errmsg,i);
    g_return_if_fail(gwf_error_add_error(error,errdat->name,errdat->type,errdat->message));
  }
}

static void
gwf_pluginlist_reg_errors_foreach (gpointer key ,gpointer value, gpointer user_data)
{
  GwfPlugin	*plugin = value;
  GwfError	*error = user_data;

  gwf_plugin_reg_errors(plugin,error);
}

/*************************/
/* Settings registration */
/*************************/

void
gwf_pluginlist_reg_settings (GwfPluginList *list,GwfSettingsList *settings)
{
  g_return_if_fail(list != NULL);
  g_return_if_fail(settings != NULL);

  g_hash_table_foreach(list->datas,gwf_pluginlist_reg_settings_foreach,(gpointer)settings);
}

void
gwf_plugin_reg_settings (GwfPlugin *plugin,GwfSettingsList *settings)
{
  GwfSettings*		setting;

  g_return_if_fail(plugin != NULL);
  g_return_if_fail(settings != NULL);

  setting = gwf_dataplug_create_settings(plugin);
  if (!gwf_settingslist_add(settings,setting))
    gwf_settings_free(setting);
}

static void
gwf_pluginlist_reg_settings_foreach (gpointer key,gpointer value,gpointer user_data)
{
  gwf_plugin_reg_settings((GwfPlugin*)value,(GwfSettingsList*)user_data);
}

static GwfSettings*
gwf_dataplug_create_settings (GwfPlugin *plugin)
{
  GwfSettings		*settings;

  g_return_val_if_fail(plugin != NULL,NULL);

  settings = gwf_settings_new(plugin->name);
  gwf_settings_set_create(settings,plugin->setting_create);

  return settings;
}

/******************************/
/* Load and save registration */
/******************************/

void
gwf_pluginlist_reg_xmlio (GwfPluginList *list,GwfXmlIO *xml_io)
{
  g_return_if_fail(list != NULL);
  g_return_if_fail(xml_io != NULL);

  g_hash_table_foreach(list->datas,gwf_pluginlist_reg_xmlio_foreach,(gpointer)xml_io);
}

void
gwf_plugin_reg_xmlio (GwfPlugin *plugin, GwfXmlIO *xml_io)
{
  g_return_if_fail(plugin != NULL);
  g_return_if_fail(xml_io != NULL);

  gwf_dataplug_create_xmlio(plugin,xml_io);
}

static void
gwf_pluginlist_reg_xmlio_foreach(gpointer key,gpointer value,gpointer user_data)
{
  gwf_plugin_reg_xmlio((GwfPlugin*)value,(GwfXmlIO*)user_data);
}

static gint
gwf_dataplug_create_xmlio (GwfPlugin *plugin,GwfXmlIO *xml_io)
{
  g_return_val_if_fail(plugin != NULL,0);

  if (!gwf_xmlio_add_loader(xml_io,plugin->name,plugin->version,plugin->href,plugin->load_plugin))
    return 0;

  if (!gwf_xmlio_add_saver(xml_io,plugin->name,plugin->version,plugin->href,plugin->save_plugin))
  {
    gwf_xmlio_del_loader(xml_io,plugin->name);
    return 0;
  }

  return 1;
}


/*****************************/
/* Project data registration */
/*****************************/

void
gwf_pluginlist_connect_projlist	(GwfPluginList *list,GwfProjlist *projlist)
{
  g_return_if_fail(list != NULL);
  g_return_if_fail(projlist != NULL);

  gtk_signal_connect (GTK_OBJECT (projlist), "project_added",
                      GTK_SIGNAL_FUNC (gwf_pluginlist_reg_project_data),
                      (gpointer)list);
}

static void	
gwf_pluginlist_reg_project_data(GwfProjlist *projlist,gchar *name,gpointer user_data)
{
  GwfProject	*project;

  project = gwf_projlist_get_project(projlist,name);
  g_hash_table_foreach(((GwfPluginList*)user_data)->datas,gwf_pluginlist_reg_data_foreach,project);
}

static void
gwf_pluginlist_reg_data_foreach(gpointer key,gpointer value,gpointer user_data)
{
  GwfProjData		*data;
  GwfDataList		*list;

  data = gwf_dataplug_create_projdata((GwfPlugin*)value,(GwfProject*)user_data);
  list = gwf_project_get_data_list((GwfProject*)user_data);

  if (!gwf_data_list_insert(list,data))
    gwf_proj_data_free(data);
}

static GwfProjData*
gwf_dataplug_create_projdata (GwfPlugin *plugin,GwfProject *project)
{
  g_return_val_if_fail(plugin != NULL,NULL);
  g_return_val_if_fail(project != NULL,NULL);

  if (plugin->new_data)
    return plugin->new_data(project);
  else
    return NULL;
}

/*****************************/
/* Project tree registration */
/*****************************/

void
gwf_pluginlist_reg_treedatas (GwfPluginList *list,GwfTreeDataList *treedatas)
{
  g_return_if_fail(list != NULL);
  g_return_if_fail(treedatas != NULL);

  g_hash_table_foreach(list->datas,gwf_pluginlist_reg_treedatas_foreach,(gpointer)treedatas);
}

void
gwf_plugin_reg_treedata (GwfPlugin *plugin,GwfTreeDataList *treedatas)
{
  GwfTreeData*		tmp;

  g_return_if_fail(plugin != NULL);
  g_return_if_fail(treedatas != NULL);

  tmp = gwf_pluginlist_create_treedata(plugin);
  g_return_if_fail(tmp != NULL);

  g_return_if_fail(gwf_treedata_list_add(treedatas,tmp));
}

static void
gwf_pluginlist_reg_treedatas_foreach(gpointer key,gpointer value,gpointer user_data)
{
  gwf_plugin_reg_treedata((GwfPlugin*)value,(GwfTreeDataList*)user_data);
}

static GwfTreeData*
gwf_pluginlist_create_treedata(GwfPlugin *plugin)
{
  GwfTreeData		*tmp;

  tmp = gwf_treedata_new(plugin->name);
  g_return_val_if_fail(tmp != NULL,NULL);

  gwf_treedata_set_attach(tmp,plugin->tree_attach);
  gwf_treedata_set_free(tmp,plugin->tree_free);

  return tmp;
}



/*******************************/
/* Source writing registration */
/*******************************/

void
gwf_pluginlist_reg_source (GwfPluginList *list,GwfSource *srcwrt)
{
  g_return_if_fail(list != NULL);
  g_return_if_fail(srcwrt != NULL);

  g_hash_table_foreach(list->datas,gwf_pluginlist_reg_source_foreach,(gpointer)srcwrt);
}

void
gwf_plugin_reg_source (GwfPlugin *plugin,GwfSource *srcwrt)
{
  GList			*tmp;
  GwfSrcPlugData	*data;

  g_return_if_fail(plugin != NULL);
  g_return_if_fail(srcwrt != NULL);

  gwf_source_add_check(srcwrt,plugin->name,plugin->source_check);

  for (tmp = plugin->source_write ; tmp ; tmp = tmp->next)
  {
    data = (GwfSrcPlugData*)tmp->data;
    gwf_source_schedule_func(srcwrt,plugin->name,data->prior,data->func,data->file);
  }
}

static void
gwf_pluginlist_reg_source_foreach(gpointer key,gpointer value,gpointer user_data)
{
  GwfPlugin		*plugin = value;
  GwfSource		*srcwrt = user_data;

  gwf_plugin_reg_source(plugin,srcwrt);
}
