/*  Gtk+ Widget Factory
 *  Copyright (C) 1999-2000  Jeroen Benckhuijsen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gwf_lib.h>
#include "gwf_projtree_elem.h"
#include "gwf_projdata_elem.h"
#include <gtk/gtk.h>

enum
{
  ELEM_ADD,
  ELEM_REMOVE,

  LAST_NODE
};

enum
{
  ELEM_NAME,
  ELEM_TYPE,
  ELEM_VALUE,
  ELEM_SETFUNC,
  ELEM_ALLOCSET,
  ELEM_SETCODE,
  ELEM_GETFUNC,
  ELEM_ALLOCGET,
  ELEM_GETCODE,

  LAST_ELEM_NODE
};

typedef struct _ElemAddData	ElemAddData;
struct _ElemAddData
{
  GwfProject	*project;
  GwfPTData	*ptdata;
};

static void		elem_tree_add_element			(GwfProject *project,gchar *name,GwfPTData *ptdata);
static void		elem_tree_remove_element		(GwfProject *project,gchar *name,GwfPTData *ptdata);
static void		elem_tree_add_foreach_element		(element *elem,gpointer user_data);

static void		elem_tree_type_changed_cb 		(GwfProject *project,gchar *name,gpointer user_data);
static void		elem_tree_name_changed_cb 		(GwfProject *project,gchar *name,gpointer user_data);
static void		elem_tree_value_changed_cb 		(GwfProject *project,gchar *name,gpointer user_data);
static void		elem_tree_setfunc_changed_cb 		(GwfProject *project,gchar *name,gpointer user_data);
static void             elem_tree_allocset_changed_cb 		(GwfProject *project,gchar *name,gpointer user_data);
static void		elem_tree_setcode_changed_cb		(GwfProject *project,gchar *name,gpointer user_data);
static void             elem_tree_getfunc_changed_cb 		(GwfProject *project,gchar *name,gpointer user_data);
static void             elem_tree_allocget_changed_cb 		(GwfProject *project,gchar *name,gpointer user_data);
static void		elem_tree_getcode_changed_cb		(GwfProject *project,gchar *name,gpointer user_data);
static gint		elem_tree_check_current_element		(GwfProject *project,gchar *name,GwfPTData *ptdata);

static GwfPTData		elemmain_pt_data[LAST_NODE] =
  {
    {NULL,	"Elements",		NULL,	"element_added",	0,	elem_tree_add_element,		NULL, NULL},
    {NULL,	"Elements",		NULL,	"element_removed",	0,	elem_tree_remove_element,	NULL, NULL}
  };

static GwfPTData		elem_pt_data[LAST_ELEM_NODE] =
  {
    {NULL,	"Name",			NULL,	"element_changed",	0,	elem_tree_name_changed_cb,	NULL, NULL},
    {NULL,	"Type",			NULL,	"element_changed",	0,	elem_tree_type_changed_cb,	NULL, NULL},
    {NULL,	"Initial value",	NULL,	"element_changed",	0,	elem_tree_value_changed_cb,	NULL, NULL},
    {NULL,	"Create set function",	NULL,	"element_changed",	0,	elem_tree_setfunc_changed_cb,	NULL, NULL},
    {NULL,	"Allocate set value",	NULL,	"element_changed",	0,	elem_tree_allocset_changed_cb,	NULL, NULL},
    {NULL,	"Set Allocation Code",	NULL,	"element_changed",	0,	elem_tree_setcode_changed_cb,	NULL, NULL},
    {NULL,	"Create get function",	NULL,	"element_changed",	0,	elem_tree_getfunc_changed_cb,	NULL, NULL},
    {NULL,	"Allocate get value",	NULL,	"element_changed",	0,	elem_tree_allocget_changed_cb,	NULL, NULL},
    {NULL,	"Get Allocation Code",	NULL,	"element_changed",	0,	elem_tree_getcode_changed_cb,	NULL, NULL}
  };

void
gwf_projtree_elem_attach (GwfTreeData *data,GwfProject *project)
{
  GwfPTData	**pt_data;
  ElemAddData	adddata;

  g_return_if_fail(data != NULL);
  g_return_if_fail(project != NULL);
  g_return_if_fail(data->tree != NULL);

  pt_data = gwf_projtree_add_new_section(data,project,elemmain_pt_data[ELEM_ADD].node,elemmain_pt_data,LAST_NODE);
  if (!pt_data)
    return;

  adddata.project = project;
  adddata.ptdata = pt_data[ELEM_ADD];
  gwf_project_element_foreach(project,elem_tree_add_foreach_element,(gpointer)&adddata);
}

static void
elem_tree_add_foreach_element(element *elem,gpointer user_data)
{
  g_return_if_fail(elem != NULL);
  g_return_if_fail(user_data != NULL);

  elem_tree_add_element(((ElemAddData*)user_data)->project,elem->name,((ElemAddData*)user_data)->ptdata);
}

static void
elem_tree_add_element(GwfProject *project,gchar *name,GwfPTData *ptdata)
{
  GtkCTreeNode	*elemnode;
  element	*elem;
  GwfPTData	**pt_data;

  elem = gwf_project_element_find(project,name);
  g_return_if_fail(elem != NULL);

  elemnode = gtk_ctree_add_node(ptdata->data->tree,ptdata->parent,"Element");
  if (!elemnode)
    return;
  gtk_ctree_node_set_row_data(ptdata->data->tree,elemnode,(gpointer)elem);
  gtk_ctree_expand(ptdata->data->tree,elemnode);

  pt_data = gwf_treedata_register_ptdata(ptdata->data,elem_pt_data,LAST_ELEM_NODE,elemnode,project);
  if (!ptdata)
    return;

  gwf_projtree_insert_string_by_ptdata(pt_data[ELEM_NAME],name);
  gwf_projtree_insert_string_by_ptdata(pt_data[ELEM_TYPE],gwf_project_element_get_type(project,name));
  gwf_projtree_insert_string_by_ptdata(pt_data[ELEM_VALUE],gwf_project_element_get_value(project,name));
  gwf_projtree_insert_bool_by_ptdata(pt_data[ELEM_SETFUNC],gwf_project_element_get_setfunc(project,name));
  gwf_projtree_insert_bool_by_ptdata(pt_data[ELEM_ALLOCSET],gwf_project_element_get_setalloc(project,name));
  gwf_projtree_insert_string_by_ptdata(pt_data[ELEM_SETCODE],gwf_project_element_get_setcode(project,name));
  gwf_projtree_insert_bool_by_ptdata(pt_data[ELEM_GETFUNC],gwf_project_element_get_getfunc(project,name));
  gwf_projtree_insert_bool_by_ptdata(pt_data[ELEM_ALLOCGET],gwf_project_element_get_getalloc(project,name));
  gwf_projtree_insert_string_by_ptdata(pt_data[ELEM_GETCODE],gwf_project_element_get_getcode(project,name));
}

static void
elem_tree_remove_element(GwfProject *project,gchar *name,GwfPTData *ptdata)
{
  element	*elem;
  GtkCTreeNode	*elemnode;

  elem = gwf_project_element_find(project,name);
  g_return_if_fail(elem != NULL);

  elemnode = gtk_ctree_find_by_row_data(ptdata->data->tree,ptdata->parent,(gpointer)elem);
  g_return_if_fail(elemnode != NULL);

  gwf_projtree_remove_node_ptdata(ptdata->data,elemnode);
}

static void
elem_tree_type_changed_cb (GwfProject *project,gchar *name,gpointer user_data)
{
  if (elem_tree_check_current_element(project,name,(GwfPTData*)user_data))
    gwf_projtree_node_string_changed_cb(project,gwf_project_element_get_type(project,name),user_data);
}

static void
elem_tree_name_changed_cb (GwfProject *project,gchar *name,gpointer user_data)
{
  if (elem_tree_check_current_element(project,name,(GwfPTData*)user_data))
    gwf_projtree_node_string_changed_cb(project,name,user_data);
}

static void
elem_tree_value_changed_cb (GwfProject *project,gchar *name,gpointer user_data)
{
  if (elem_tree_check_current_element(project,name,(GwfPTData*)user_data))
    gwf_projtree_node_string_changed_cb(project,gwf_project_element_get_value(project,name),user_data);
}

static void
elem_tree_setfunc_changed_cb (GwfProject *project,gchar *name,gpointer user_data)
{
  if (elem_tree_check_current_element(project,name,(GwfPTData*)user_data))
    gwf_projtree_node_bool_changed_cb(project,gwf_project_element_get_setfunc(project,name),user_data);
}

static void
elem_tree_allocset_changed_cb (GwfProject *project,gchar *name,gpointer user_data)
{
  if (elem_tree_check_current_element(project,name,(GwfPTData*)user_data))
    gwf_projtree_node_bool_changed_cb(project,gwf_project_element_get_setalloc(project,name),user_data);
}

static void
elem_tree_setcode_changed_cb (GwfProject *project,gchar *name,gpointer user_data)
{
  if (elem_tree_check_current_element(project,name,(GwfPTData*)user_data))
    gwf_projtree_node_string_changed_cb(project,gwf_project_element_get_setcode(project,name),user_data);
}

static void
elem_tree_getfunc_changed_cb (GwfProject *project,gchar *name,gpointer user_data)
{
  if (elem_tree_check_current_element(project,name,(GwfPTData*)user_data))
    gwf_projtree_node_bool_changed_cb(project,gwf_project_element_get_getfunc(project,name),user_data);
}

static void
elem_tree_allocget_changed_cb (GwfProject *project,gchar *name,gpointer user_data)
{
  if (elem_tree_check_current_element(project,name,(GwfPTData*)user_data))
    gwf_projtree_node_bool_changed_cb(project,gwf_project_element_get_getalloc(project,name),user_data);
}

static void
elem_tree_getcode_changed_cb (GwfProject *project,gchar *name,gpointer user_data)
{
  if (elem_tree_check_current_element(project,name,(GwfPTData*)user_data))
    gwf_projtree_node_string_changed_cb(project,gwf_project_element_get_getcode(project,name),user_data);
}

static gint
elem_tree_check_current_element(GwfProject *project,gchar *name,GwfPTData *ptdata)
{
  element	*elem;
  GtkCTreeNode	*elemnode;

  elem = gwf_project_element_find(project,name);
  g_return_val_if_fail(elem != NULL,0);

  elemnode = gtk_ctree_find_by_row_data(ptdata->data->tree,ptdata->parent,(gpointer)elem);
  return (elemnode == ptdata->parent);
}
