/*  Gtk+ Widget Factory
 *  Copyright (C) 1999-2000  Jeroen Benckhuijsen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gwf_lib.h>
#include "gwf_source_elem.h"
#include "gwf_projdata_elem.h"
#include <gtk/gtk.h>

static void		check_elem_foreach		(element *elem,gpointer user_data);
static void		write_elem_type_foreach		(element *elem,gpointer user_data);
static void		write_elem_func_decl_foreach	(element *elem,gpointer user_data);
static void		write_elem_init_foreach		(element *elem,gpointer user_data);
static void		write_elem_func_foreach		(element *elem,gpointer user_data);
static void		write_setget_code		(GString *src,gchar *code);

/*****************************/
/* Prewriting check function */
/*****************************/
gint
gwf_source_elem_check (GwfProject *project,GwfError *error)
{
  gint		result = 1;

  g_return_val_if_fail(project != NULL,0);
  g_return_val_if_fail(error != NULL,0);

  gwf_project_element_foreach(project,check_elem_foreach,(gpointer)&result);

  if (!result)
    gwf_error_raise_error(error,"Source : Elem : Invalid elem");

  return result;
}

static void
check_elem_foreach(element *elem,gpointer user_data)
{
  gint		*result = user_data;

  if (!elem->type) *result = 0;
  if (!elem->name) *result = 0;
}

/*************************/
/* HFile write functions */
/*************************/
void
gwf_source_elem_write_elem_types (GwfProject *project,GString *src)
{
  gchar		*structname;
  gchar		*tmp;
  gint		pos;
  GwfSource	srcwrt;

  g_return_if_fail(project != NULL);
  g_return_if_fail(src != NULL);

  structname = gwf_source_get_widname_as_struct(project);
  tmp = g_strdup_printf("struct _%s\n{",structname);

  pos = gwf_source_find_text(src,tmp);
  pos = gwf_source_find_text_from_pos(src,pos,"parent");
  pos += 8;

  g_free(structname);
  g_free(tmp);

  srcwrt.result = pos;
  srcwrt.project = project;
  srcwrt.src = src;

  srcwrt.result = gwf_source_insert(src,srcwrt.result,"\n");
  gwf_project_element_foreach(project,write_elem_type_foreach,(gpointer)&srcwrt);
}

static void
write_elem_type_foreach (element *elem,gpointer user_data)
{
  GwfSource	*srcwrt = user_data;
  gchar		*structname;
  gint		maxlen;

  structname = gwf_source_get_parent_as_struct(srcwrt->project);
  maxlen = strlen(structname) + 8 - (strlen(structname) % 8);

  srcwrt->result = gwf_source_insert_tabs_after_printf(srcwrt->src,srcwrt->result,maxlen,"  %s",elem->type);
  srcwrt->result = gwf_source_insert_printf(srcwrt->src,srcwrt->result,"%s;\n",elem->name);

  g_free(structname);
}

void
gwf_source_elem_write_func_decl (GwfProject *project,GString *src)
{
  GwfSource	srcwrt;

  g_return_if_fail(project != NULL);
  g_return_if_fail(src != NULL);

  srcwrt.project = project;
  srcwrt.src = src;

  gwf_project_element_foreach(project,write_elem_func_decl_foreach,(gpointer)&srcwrt);
  g_string_append(src,"\n\n");
}

static void
write_elem_func_decl_foreach (element *elem,gpointer user_data)
{
  GwfSource	*srcwrt = user_data;
  gchar		*structname;
  gchar		*funcname,*varname;
  gint		maxlen;

  structname = gwf_source_get_widname_as_struct(srcwrt->project);
  maxlen = strlen(structname);

  funcname = gwf_source_get_widname_as_func(srcwrt->project);
  varname = gwf_source_get_widname_as_var(srcwrt->project);

  if (elem->set_func)
  {
    gwf_source_add_tabs_after(srcwrt->src,"void",maxlen);
    gwf_source_add_tabs_after_until_printf(srcwrt->src,48, "%s_set_%s",funcname,elem->name);
    g_string_sprintfa(srcwrt->src,"(%s *%s,%s %s);\n",structname,varname,elem->type,elem->name);
  }

  if (elem->get_func)
  {
    gwf_source_add_tabs_after_printf(srcwrt->src,maxlen,"%s",elem->type);
    gwf_source_add_tabs_after_until_printf(srcwrt->src,48, "%s_get_%s",funcname,elem->name);
    g_string_sprintfa(srcwrt->src,"(%s *%s);\n",structname,varname);
  }

  if (elem->set_func || elem->get_func)
    g_string_append_c(srcwrt->src,'\n');

  g_free(structname);
  g_free(varname);
  g_free(funcname);
}

/*************************/
/* CFile write functions */
/*************************/
void
gwf_source_elem_write_init_func (GwfProject *project,GString *src)
{
  gchar		*structname,*funcname;
  gchar		*varname,*tmp;
  GwfSource	srcwrt;

  g_return_if_fail(project != NULL);
  g_return_if_fail(src != NULL);

  structname = gwf_source_get_widname_as_struct(project);
  funcname = gwf_source_get_widname_as_func(project);
  varname = gwf_source_get_widname_as_var(project);
  tmp = g_strdup_printf("static void\n%s_init (%s *%s)\n",funcname,structname,varname);
  g_free(structname);
  g_free(funcname);
  g_free(varname);

  srcwrt.result = gwf_source_find_text(src,tmp);
  srcwrt.result += strlen(tmp) + 2;
  g_free(tmp);

  srcwrt.project = project;
  srcwrt.src = src;

  gwf_project_element_foreach(project,write_elem_init_foreach,(gpointer)&srcwrt);
}

static void
write_elem_init_foreach (element *elem,gpointer user_data)
{
  gchar		*varname;
  GwfSource	*srcwrt = user_data;

  varname = gwf_source_get_widname_as_var(srcwrt->project);

  if (elem->value)
    srcwrt->result = gwf_source_insert_printf(srcwrt->src,srcwrt->result,"  %s->%s = %s;\n",varname,elem->name,elem->value);

  g_free(varname);
}

void
gwf_source_elem_write_funcs (GwfProject *project,GString *src)
{
  GwfSource	srcwrt;

  g_return_if_fail(project != NULL);
  g_return_if_fail(src != NULL);

  srcwrt.project = project;
  srcwrt.src = src;

  g_string_append_c(src,'\n');

  gwf_project_element_foreach(project,write_elem_func_foreach,(gpointer)&srcwrt);

  g_string_append_c(src,'\n');
}

static void
write_elem_func_foreach(element *elem,gpointer user_data)
{
  GwfSource	*srcwrt = user_data;
  gchar		*varname;
  gchar		*funcname;
  gchar		*structname;

  structname = gwf_source_get_widname_as_struct(srcwrt->project);
  funcname = gwf_source_get_widname_as_func(srcwrt->project);
  varname = gwf_source_get_widname_as_var(srcwrt->project);

  if (elem->set_func)
  {
    g_string_append(srcwrt->src,"void\n");
    g_string_sprintfa(srcwrt->src,"%s_set_%s ",funcname,elem->name);
    g_string_sprintfa(srcwrt->src,"(%s *%s,%s %s)\n{\n",structname,varname,elem->type,elem->name);
    g_string_sprintfa(srcwrt->src,"  g_return_if_fail(%s != NULL);\n",varname);

    if (elem->alloc_set_func)
      write_setget_code(srcwrt->src,elem->set_code);
    else
      g_string_sprintfa(srcwrt->src,"  %s->%s = %s;\n",varname,elem->name,elem->name);

    g_string_append(srcwrt->src,"}\n\n");
  }

  if (elem->get_func)
  {
    g_string_sprintfa(srcwrt->src,"%s\n",elem->type);
    g_string_sprintfa(srcwrt->src,"%s_get_%s ",funcname,elem->name);
    g_string_sprintfa(srcwrt->src,"(%s *%s)\n{\n",structname,varname);
    g_string_sprintfa(srcwrt->src,"  g_return_val_if_fail(%s != NULL,%s);\n",varname,(elem->alloc_get_func) ? "NULL" : "0");

    if (elem->alloc_get_func)
      write_setget_code(srcwrt->src,elem->get_code);
    else
      g_string_sprintfa(srcwrt->src,"  return %s->%s;\n",varname,elem->name);

    g_string_append(srcwrt->src,"}\n\n");
  }

  g_free(structname);
  g_free(varname);
  g_free(funcname);
}

static void
write_setget_code(GString *src,gchar *code)
{
  g_return_if_fail(src != NULL);

  if (!code)
  {
    g_string_append(src, "  #warning Enter allocation code here\n");
    return;
  }

  g_string_append(src,"  ");
  for ( ; *code != '\0' ; code++)
  {
    if (*code != '\n')
      g_string_append_c(src,*code);
    else
      g_string_append(src,"\n  ");
  }
  g_string_append_c(src,'\n');
}
