/*  Gtk+ Widget Factory
 *  Copyright (C) 1999-2000  Jeroen Benckhuijsen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gwf_lib.h>
#include <gtk/gtk.h>
#include "gwf_projdata_elem.h"
#include "gwf_widprop_elem.h"

static void		gwf_widpropelem_class_init	(GwfWidPropElemClass	*klass);
static void		gwf_widpropelem_init		(GwfWidPropElem		*wpelem);
static void		gwf_widpropelem_destroy		(GtkObject		*object);

static void		gwf_widpropelem_connect		(GwfProject *project,GwfWidPropElem *wpelem);
static void		gwf_widpropelem_disconnect	(GwfProject *project,GwfWidPropElem *wpelem);
static void		gwf_widpropelem_update		(GwfWidPropElem *wpelem);

/**************************************************/
/* Callbacks for the interaction with the project */
/**************************************************/
static void curproj_changed_cb		(GwfProjlist *projlist, gchar *name,gpointer user_data);
static void project_added_cb		(GwfProjlist *projlist, gchar *name,gpointer user_data);
static void element_added_cb 		(GwfProject *project,gchar *name, gpointer user_data);
static void element_changed_cb 		(GwfProject *project,gchar *name, gpointer user_data);
static void element_removed_cb 		(GwfProject *project,gchar *name, gpointer user_data);

/**************************/
/* GUI callback functions */
/**************************/
static void	on_clElem_select_row		(GtkCList *clist,gint row,gint column,GdkEvent *event,gpointer user_data);
static void	on_btnElemAdd_clicked		(GtkButton *button,gpointer user_data);
static void	on_btnElemDelete_clicked	(GtkButton *button,gpointer user_data);
static void	on_btnElemClear_clicked		(GtkButton *button,gpointer user_data);
static void	on_btnSetFunc_toggled		(GtkToggleButton *button,gpointer user_data);
static void	on_btnGetFunc_toggled		(GtkToggleButton *button,gpointer user_data);
static void	on_btnAllocSet_toggled		(GtkToggleButton *button,gpointer user_data);
static void	on_btnAllocGet_toggled		(GtkToggleButton *button,gpointer user_data);

/****************/
/* GUI Creation */
/****************/
static void		element_add_clist_foreach	(element *elem,gpointer user_data);
static void		gwf_widpropelem_interface 	(GwfWidPropElem *wpelem);
static GtkWidget*	gwf_widpropelem_elemlist 	(GwfWidPropElem *wpelem);
static GtkWidget*	gwf_widpropelem_elem_entry 	(GwfWidPropElem *wpelem);
static GtkWidget*	gwf_widpropelem_buttons 	(GwfWidPropElem *wpelem);


guint
gwf_widpropelem_get_type (void)
{
  static guint gwf_widpropelem_type = 0;

  if (!gwf_widpropelem_type)
    {
      GtkTypeInfo gwf_widpropelem_info =
      {
	"GwfWidPropElem",
	sizeof (GwfWidPropElem),
	sizeof (GwfWidPropElemClass),
	(GtkClassInitFunc) gwf_widpropelem_class_init,
	(GtkObjectInitFunc) gwf_widpropelem_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL,
      };

      gwf_widpropelem_type = gtk_type_unique (gtk_vbox_get_type(), &gwf_widpropelem_info);
    }

  return gwf_widpropelem_type;
}

static void		
gwf_widpropelem_class_init (GwfWidPropElemClass *klass)
{
  ((GtkObjectClass *)klass)->destroy = gwf_widpropelem_destroy;
}

static void
gwf_widpropelem_init (GwfWidPropElem *wpelem)
{
  GtkStyle	*style;
  GdkColor	*color = NULL;

  gwf_widpropelem_interface(wpelem);

  style = gtk_widget_get_style(GTK_WIDGET(wpelem->elemlist));
  color = &(style->base[GTK_STATE_NORMAL]);

  wpelem->item_set = gwf_gdkpixmap_create_with_transp(GTK_WIDGET(wpelem->elemlist),item_set_xpm,color);
}

void
gwf_widpropelem_construct(GwfWidPropElem *wpelem,GwfProjlist *projlist)
{
  wpelem->projlist = projlist;
  gwf_projlist_project_foreach(projlist,(GwfProjlistForeachFunc)gwf_widpropelem_connect,(gpointer)wpelem);
  gwf_widpropelem_update(wpelem);

  gtk_signal_connect (GTK_OBJECT (projlist), "curproj_changed",
                      GTK_SIGNAL_FUNC(curproj_changed_cb),
                      (gpointer)wpelem);  
  gtk_signal_connect (GTK_OBJECT (projlist), "project_added",
                      GTK_SIGNAL_FUNC (project_added_cb),
                      (gpointer)wpelem);
}

GtkWidget *
gwf_widpropelem_new(GwfProjlist *projlist)
{
  GwfWidPropElem	*wpelem;
  
  wpelem = GWF_WIDPROP_ELEM(gtk_type_new (gwf_widpropelem_get_type()));
  gwf_widpropelem_construct(wpelem,projlist);
  return GTK_WIDGET ( wpelem);
}

static void
gwf_widpropelem_destroy (GtkObject *object)
{
  gwf_projlist_project_foreach(GWF_WIDPROP_ELEM(object)->projlist,
     			       (GwfProjlistForeachFunc)gwf_widpropelem_disconnect,
                               (gpointer)object);
  gtk_signal_disconnect_by_data(GTK_OBJECT(GWF_WIDPROP_ELEM(object)->projlist),(gpointer)object);
  if (GWF_WIDPROP_ELEM(object)->item_set)
    gdk_pixmap_unref(GWF_WIDPROP_ELEM(object)->item_set);
}


static void
gwf_widpropelem_update		(GwfWidPropElem *wpelem)
{
  GwfProject	*project;
  
  gtk_clist_clear(wpelem->elemlist);

  gtk_entry_set_text(wpelem->type,"");
  gtk_entry_set_text(wpelem->name,"");
  gtk_entry_set_text(wpelem->value,"");
  gtk_toggle_button_set_active(wpelem->setfunc,FALSE);
  gtk_toggle_button_set_active(wpelem->getfunc,FALSE);
  gtk_toggle_button_set_active(wpelem->allocset,FALSE);
  gtk_toggle_button_set_active(wpelem->allocget,FALSE);

  gtk_widget_set_sensitive(GTK_WIDGET(wpelem->allocset),FALSE);
  gtk_widget_set_sensitive(GTK_WIDGET(wpelem->setcode),FALSE);

  gtk_widget_set_sensitive(GTK_WIDGET(wpelem->allocget),FALSE);
  gtk_widget_set_sensitive(GTK_WIDGET(wpelem->getcode),FALSE);

  gtk_clist_freeze(wpelem->elemlist);

  project = gwf_projlist_get_current(wpelem->projlist);
  if (project)
    gwf_project_element_foreach(project,element_add_clist_foreach,(gpointer)wpelem);

  gtk_clist_thaw(wpelem->elemlist);
}

static void
gwf_widpropelem_disconnect(GwfProject *project,GwfWidPropElem *wpelem)
{
  gtk_signal_disconnect_by_data(GTK_OBJECT(project),(gpointer)wpelem);
}

static void
gwf_widpropelem_connect(GwfProject *project,GwfWidPropElem *wpelem)
{
  gtk_signal_connect (GTK_OBJECT (project), "element_added",
     		      GTK_SIGNAL_FUNC (element_added_cb),
                      (gpointer)wpelem);
  gtk_signal_connect (GTK_OBJECT (project), "element_changed",
     		      GTK_SIGNAL_FUNC (element_changed_cb),
                      (gpointer)wpelem);
  gtk_signal_connect (GTK_OBJECT (project), "element_removed",
     		      GTK_SIGNAL_FUNC (element_removed_cb),
                      (gpointer)wpelem);
}

void
gwf_widpropelem_set_itemset(GwfWidPropElem* wpelem,GdkPixmap *itemset)
{
  g_return_if_fail(wpelem != NULL);
  wpelem->item_set = itemset;
}

/**************************************************/
/* Callbacks for the interaction with the project */
/**************************************************/

static void 
curproj_changed_cb (GwfProjlist *projlist, gchar *name,gpointer user_data)
{
  gwf_widpropelem_update(GWF_WIDPROP_ELEM(user_data));
}

static void
project_added_cb	(GwfProjlist *projlist, gchar *name,gpointer user_data)
{
  GwfProject		*project;

  project = gwf_projlist_get_project(projlist,name);
  gwf_widpropelem_connect(project,GWF_WIDPROP_ELEM(user_data));  
}

static void
element_added_cb (GwfProject *project,gchar *name, gpointer user_data)
{
  GwfWidPropElem 	*wpelem = user_data; 
  gchar			*temp[7] = {NULL};
  gint			row;
  
  if (!gwf_projlist_project_is_current(wpelem->projlist,project))
    return;
  
  temp[0] = NULL_TO_EMPTY(gwf_project_element_get_type(project,name));
  temp[1] = name;
  temp[2] = NULL_TO_EMPTY(gwf_project_element_get_value(project,name));

  gtk_clist_freeze(wpelem->elemlist);

  row = gtk_clist_append(wpelem->elemlist,temp);

  if (gwf_project_element_get_setfunc(project,name))
    gtk_clist_set_pixmap(wpelem->elemlist,row,3,wpelem->item_set,NULL);
  else
    gtk_clist_set_text(wpelem->elemlist,row,3,"");

  if (gwf_project_element_get_setalloc(project,name))
    gtk_clist_set_pixmap(wpelem->elemlist,row,4,wpelem->item_set,NULL);
  else
    gtk_clist_set_text(wpelem->elemlist,row,4,"");

  if (gwf_project_element_get_getfunc(project,name))
    gtk_clist_set_pixmap(wpelem->elemlist,row,5,wpelem->item_set,NULL);
  else
    gtk_clist_set_text(wpelem->elemlist,row,5,"");

  if (gwf_project_element_get_getalloc(project,name))
    gtk_clist_set_pixmap(wpelem->elemlist,row,6,wpelem->item_set,NULL);
  else
    gtk_clist_set_text(wpelem->elemlist,row,6,"");

  gtk_clist_select_row(wpelem->elemlist,row,0);

  gtk_clist_thaw(wpelem->elemlist);
}

static void
element_changed_cb (GwfProject *project,gchar *name, gpointer user_data)
{
  GwfWidPropElem 	*wpelem = user_data; 
  gint			row;

  if (!gwf_projlist_project_is_current(wpelem->projlist,project))
    return;
  
  if (!wpelem->elemlist->selection)
    g_warning("Can't find signal in clist");  
    
  row = GPOINTER_TO_INT(wpelem->elemlist->selection->data);
    
  gtk_clist_freeze(wpelem->elemlist);

  gtk_clist_set_text(wpelem->elemlist,row,0,NULL_TO_EMPTY(gwf_project_element_get_type(project,name)));
  gtk_clist_set_text(wpelem->elemlist,row,1,name);
  gtk_clist_set_text(wpelem->elemlist,row,2,NULL_TO_EMPTY(gwf_project_element_get_value(project,name)));

  if (gwf_project_element_get_setfunc(project,name))
    gtk_clist_set_pixmap(wpelem->elemlist,row,3,wpelem->item_set,NULL);
  else
    gtk_clist_set_text(wpelem->elemlist,row,3,"");

  if (gwf_project_element_get_setalloc(project,name))
    gtk_clist_set_pixmap(wpelem->elemlist,row,4,wpelem->item_set,NULL);
  else
    gtk_clist_set_text(wpelem->elemlist,row,4,"");

  if (gwf_project_element_get_getfunc(project,name))
    gtk_clist_set_pixmap(wpelem->elemlist,row,5,wpelem->item_set,NULL);
  else
    gtk_clist_set_text(wpelem->elemlist,row,5,"");

  if (gwf_project_element_get_getalloc(project,name))
    gtk_clist_set_pixmap(wpelem->elemlist,row,6,wpelem->item_set,NULL);
  else
    gtk_clist_set_text(wpelem->elemlist,row,6,"");

  gtk_clist_thaw(wpelem->elemlist);
}

static void
element_removed_cb (GwfProject *project,gchar *name, gpointer user_data)
{
  GwfWidPropElem 	*wpelem = user_data; 
  gint			row;
  
  if (!gwf_projlist_project_is_current(wpelem->projlist,project))
    return;
  
  row = gtk_clist_get_row_by_text (wpelem->elemlist,1,name);
  if (row == -1)
    g_warning("Can't find signal in clist");
  
  gtk_clist_remove(wpelem->elemlist,row);
  gtk_clist_unselect_all(wpelem->elemlist);  
}


/**************************/
/* GUI callback functions */
/**************************/
static void
on_clElem_select_row                   (GtkCList        *clist,
                                        gint             row,
                                        gint             column,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  GwfWidPropElem 	*wpelem = user_data; 
  gchar			*name = NULL;
  gchar			*temp = NULL;
  GwfProject		*project;
  gchar			*code;
  gint			pos;
  gboolean		state1,state2;
  
  project = gwf_projlist_get_current(wpelem->projlist);
  if (!project)
    return;  

  if (gtk_clist_get_text(clist,row,0,&temp))
    gtk_entry_set_text(wpelem->type,temp);
    
  if (gtk_clist_get_text(clist,row,1,&name))
    gtk_entry_set_text(wpelem->name,name);

  if (gtk_clist_get_text(clist,row,2,&temp))
    gtk_entry_set_text(wpelem->value,temp);


  state1 = gwf_project_element_get_setfunc(project,name);
  state2 = gwf_project_element_get_setalloc(project,name);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wpelem->setfunc),state1);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wpelem->allocset),state2);
  gtk_widget_set_sensitive(GTK_WIDGET(wpelem->allocset),state1);
  gtk_widget_set_sensitive(GTK_WIDGET(wpelem->setcode),state1 && state2);
  gtk_editable_delete_text(GTK_EDITABLE(wpelem->setcode),0,-1);
  pos = 0;
  code = gwf_project_element_get_setcode(project,name);
  if (code)
    gtk_editable_insert_text(GTK_EDITABLE(wpelem->setcode),code,strlen(code),&pos);


  state1 = gwf_project_element_get_getfunc(project,name);
  state2 = gwf_project_element_get_getalloc(project,name);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wpelem->getfunc),state1);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wpelem->allocget),state2);
  gtk_widget_set_sensitive(GTK_WIDGET(wpelem->allocget),state1);
  gtk_widget_set_sensitive(GTK_WIDGET(wpelem->getcode),state1 && state2);
  gtk_editable_delete_text(GTK_EDITABLE(wpelem->getcode),0,-1);
  pos = 0;
  code = gwf_project_element_get_getcode(project,name);
  if (code)
    gtk_editable_insert_text(GTK_EDITABLE(wpelem->getcode),code,strlen(code),&pos);
}

static void
on_btnElemAdd_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
  GwfWidPropElem 	*wpelem = user_data; 
  GwfProject		*project;
  element		elem;

  project = gwf_projlist_get_current(wpelem->projlist);
  if (!project)
    return;  
  
  elem.name = gtk_editable_get_chars(GTK_EDITABLE(wpelem->name),0,-1);
  if (!EMPTY_TO_NULL(elem.name))
  {
    g_free(elem.name);
    return;
  }

  elem.type = gtk_editable_get_chars(GTK_EDITABLE(wpelem->type),0,-1);
  if (!EMPTY_TO_NULL(elem.type))
  {
    g_free(elem.type);
    elem.type = NULL;
  }

  elem.value = gtk_editable_get_chars(GTK_EDITABLE(wpelem->value),0,-1);
  if (!EMPTY_TO_NULL(elem.value))
  {
    g_free(elem.value);
    elem.value = NULL;
  }

  elem.set_code = gtk_editable_get_chars(GTK_EDITABLE(wpelem->setcode),0,-1);
  if (!EMPTY_TO_NULL(elem.set_code))
  {
    g_free(elem.set_code);
    elem.set_code = NULL;
  }

  elem.get_code = gtk_editable_get_chars(GTK_EDITABLE(wpelem->getcode),0,-1);
  if (!EMPTY_TO_NULL(elem.get_code))
  {
    g_free(elem.get_code);
    elem.get_code = NULL;
  }

  elem.set_func = gtk_toggle_button_get_active(wpelem->setfunc);
  elem.alloc_set_func = gtk_toggle_button_get_active(wpelem->allocset);

  elem.get_func = gtk_toggle_button_get_active(wpelem->getfunc);
  elem.alloc_get_func = gtk_toggle_button_get_active(wpelem->allocget);

  gwf_project_add_element(project,&elem);

  gtk_clist_select_row(wpelem->elemlist,gtk_clist_get_row_by_text(wpelem->elemlist,1,elem.name),0);
  g_free(elem.name);
  g_free(elem.type);
  g_free(elem.value);
  g_free(elem.set_code);
  g_free(elem.get_code);
}


static void
on_btnElemDelete_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
  GwfWidPropElem 	*wpelem = user_data; 
  gchar			*temp;
  GwfProject		*project;
  
  project = gwf_projlist_get_current(wpelem->projlist);
  if (!project)
    return;  
    
  temp = gtk_clist_get_sel_row_col_text(wpelem->elemlist,1);
  if (!temp)
    return; 
    
  gwf_project_del_element(project,temp);
  g_free(temp);
}


static void
on_btnElemClear_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
  GwfWidPropElem 	*wpelem = user_data; 
    
  gtk_entry_set_text(wpelem->type,"");
  gtk_entry_set_text(wpelem->name,"");
  gtk_entry_set_text(wpelem->value,"");
  gtk_toggle_button_set_active(wpelem->setfunc,FALSE);
  gtk_toggle_button_set_active(wpelem->allocset,FALSE);
  gtk_toggle_button_set_active(wpelem->getfunc,FALSE);
  gtk_toggle_button_set_active(wpelem->allocget,FALSE);
  gtk_editable_delete_text(GTK_EDITABLE(wpelem->setcode),0,-1);
  gtk_editable_delete_text(GTK_EDITABLE(wpelem->getcode),0,-1);
}

static void
on_btnElemUpdate_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
  GwfWidPropElem 	*wpelem = user_data; 
  gchar			*type = NULL;
  gchar			*name = NULL;
  gchar			*value = NULL;
  gchar			*newname = NULL;
  gboolean		set_func = FALSE;
  gboolean		get_func = FALSE;
  gchar			*set_code = NULL;
  gboolean		alloc_set = FALSE;
  gboolean		alloc_get = FALSE;
  gchar			*get_code = NULL;
  GwfProject		*project;

  project = gwf_projlist_get_current(wpelem->projlist);
  if (!project)
    return;  
  
  name = gtk_clist_get_sel_row_col_text(wpelem->elemlist,1);
  if (!name)
    return;
 
  newname = gtk_editable_get_chars(GTK_EDITABLE(wpelem->name),0,-1);
  if (!EMPTY_TO_NULL(newname))
  {
    g_free(newname);
    return;
  }

  type = gtk_editable_get_chars(GTK_EDITABLE(wpelem->type),0,-1);
  if (!EMPTY_TO_NULL(type))
  {
    g_free(type);
    type = NULL;
  }

  value = gtk_editable_get_chars(GTK_EDITABLE(wpelem->value),0,-1);
  if (!EMPTY_TO_NULL(value))
  {
    g_free(value);
    value = NULL;
  }

  set_code = gtk_editable_get_chars(GTK_EDITABLE(wpelem->setcode),0,-1);
  if (!EMPTY_TO_NULL(set_code))
  {
    g_free(set_code);
    set_code = NULL;
  }

  get_code = gtk_editable_get_chars(GTK_EDITABLE(wpelem->getcode),0,-1);
  if (!EMPTY_TO_NULL(get_code))
  {
    g_free(get_code);
    get_code = NULL;
  }

  set_func = gtk_toggle_button_get_active(wpelem->setfunc);
  alloc_set = gtk_toggle_button_get_active(wpelem->allocset);
  get_func = gtk_toggle_button_get_active(wpelem->getfunc);
  alloc_get = gtk_toggle_button_get_active(wpelem->allocget);
  
  gwf_project_element_set_type(project,name,type);
  gwf_project_element_set_value(project,name,value);
  gwf_project_element_set_setfunc(project,name,set_func,alloc_set);
  gwf_project_element_set_setcode(project,name,set_code);
  gwf_project_element_set_getfunc(project,name,get_func,alloc_get);
  gwf_project_element_set_getcode(project,name,get_code);
  gwf_project_element_set_name(project,name,newname);
  
  g_free(type);
  g_free(value);
  g_free(newname);
  g_free(name);
  g_free(set_code);
  g_free(get_code);
}

static void
on_btnSetFunc_toggled			(GtkToggleButton	*button,
					 gpointer		user_data)
{
  GwfWidPropElem 	*wpelem = user_data;
  gboolean		state1,state2;


  state1 = gtk_toggle_button_get_active(button);
  state2 = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wpelem->allocset));

  gtk_widget_set_sensitive(GTK_WIDGET(wpelem->allocset),state1);
  gtk_widget_set_sensitive(GTK_WIDGET(wpelem->setcode),state1 && state2);
}

static void
on_btnGetFunc_toggled			(GtkToggleButton	*button,
					 gpointer		user_data)
{
  GwfWidPropElem 	*wpelem = user_data;
  gboolean		state1,state2;

  state1= gtk_toggle_button_get_active(button);
  state2 = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wpelem->allocget));

  gtk_widget_set_sensitive(GTK_WIDGET(wpelem->allocget),state1);
  gtk_widget_set_sensitive(GTK_WIDGET(wpelem->getcode),state1 && state2);
}

static void
on_btnAllocSet_toggled			(GtkToggleButton	*button,
					 gpointer		user_data)
{
  GwfWidPropElem 	*wpelem = user_data;

  gtk_widget_set_sensitive(GTK_WIDGET(wpelem->setcode),gtk_toggle_button_get_active(button));
}

static void
on_btnAllocGet_toggled			(GtkToggleButton	*button,
					 gpointer		user_data)
{
  GwfWidPropElem 	*wpelem = user_data;

  gtk_widget_set_sensitive(GTK_WIDGET(wpelem->getcode),gtk_toggle_button_get_active(button));
}

/****************/
/* GUI Creation */
/****************/
static void
element_add_clist_foreach		(element	*elem,
   					 gpointer	user_data)
{
  GwfWidPropElem 	*wpelem = user_data; 
  gchar			*temp[7] = {NULL};
  gint			row;
  
  temp[0] = elem->type;
  temp[1] = elem->name;
  temp[2] = elem->value;

  row = gtk_clist_append(wpelem->elemlist,temp);

  if (elem->set_func)
    gtk_clist_set_pixmap(wpelem->elemlist,row,3,wpelem->item_set,NULL);
  else
    gtk_clist_set_text(wpelem->elemlist,row,3,"");

  if (elem->alloc_set_func)
    gtk_clist_set_pixmap(wpelem->elemlist,row,4,wpelem->item_set,NULL);
  else
    gtk_clist_set_text(wpelem->elemlist,row,4,"");

  if (elem->get_func)
    gtk_clist_set_pixmap(wpelem->elemlist,row,5,wpelem->item_set,NULL);
  else
    gtk_clist_set_text(wpelem->elemlist,row,5,"");

  if (elem->alloc_get_func)
    gtk_clist_set_pixmap(wpelem->elemlist,row,6,wpelem->item_set,NULL);
  else
    gtk_clist_set_text(wpelem->elemlist,row,6,"");
}


static void
gwf_widpropelem_interface (GwfWidPropElem *wpelem)
{
  GtkWidget *table;

  table = gtk_table_new (3, 1, FALSE);
  gtk_widget_show (table);
  gtk_box_pack_start(GTK_BOX(wpelem),table,TRUE,TRUE,0);

  gtk_table_attach (GTK_TABLE (table), gwf_widpropelem_elemlist (wpelem), 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);

  gtk_table_attach (GTK_TABLE (table), gwf_widpropelem_elem_entry (wpelem), 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);

  gtk_table_attach (GTK_TABLE (table), gwf_widpropelem_buttons(wpelem), 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 5, 5);
}

static GtkWidget*
gwf_widpropelem_elemlist (GwfWidPropElem *wpelem)
{
  GtkWidget	*scrollwin;
  GtkWidget	*clist;
  GtkWidget	*label;


  scrollwin = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrollwin);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrollwin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  clist = gtk_clist_new (7);
  wpelem->elemlist = GTK_CLIST(clist);
  gtk_widget_show (clist);
  gtk_container_add (GTK_CONTAINER (scrollwin), clist);
  gtk_clist_set_column_width (GTK_CLIST (clist), 0, 80);
  gtk_clist_set_column_width (GTK_CLIST (clist), 1, 80);
  gtk_clist_set_column_width (GTK_CLIST (clist), 2, 80);
  gtk_clist_set_column_width (GTK_CLIST (clist), 3, 80);
  gtk_clist_set_column_width (GTK_CLIST (clist), 4, 80);
  gtk_clist_set_column_width (GTK_CLIST (clist), 5, 80);
  gtk_clist_set_column_width (GTK_CLIST (clist), 6, 80);
  gtk_clist_column_titles_show (GTK_CLIST (clist));

  label = gtk_label_new ("Type");
  gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (clist), 0, label);

  label = gtk_label_new ("Name");
  gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (clist), 1, label);

  label = gtk_label_new ("Value");
  gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (clist), 2, label);

  label = gtk_label_new ("Set func");
  gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (clist), 3, label);

  label = gtk_label_new ("Allocate set");
  gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (clist), 4, label);

  label = gtk_label_new ("Get func");
  gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (clist), 5, label);

  label = gtk_label_new ("Allocate get");
  gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (clist), 6, label);

  gtk_signal_connect (GTK_OBJECT (clist), "select_row",
                      GTK_SIGNAL_FUNC (on_clElem_select_row),
                      (gpointer)wpelem);
  return scrollwin;
}

static GtkWidget*
gwf_widpropelem_elem_entry (GwfWidPropElem *wpelem)
{
  GtkWidget	*table;
  GtkWidget	*entry;
  GtkWidget	*label;
  GtkWidget	*table2;
  GtkWidget	*frame;
  GtkWidget	*text;
  GtkWidget	*checkbutton;
  GtkWidget	*scrollwin;

  table = gtk_table_new (5, 2, FALSE);
  gtk_widget_show (table);

  label = gtk_label_new ("Type :");
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 5, 5);
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
  
  entry = gtk_entry_new ();
  wpelem->type = GTK_ENTRY(entry);
  gtk_widget_show (entry);
  gtk_table_attach (GTK_TABLE (table), entry, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
                    
                    
  label = gtk_label_new ("Name :");
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 5, 5);
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
  
  entry = gtk_entry_new ();
  wpelem->name = GTK_ENTRY(entry);
  gtk_widget_show (entry);
  gtk_table_attach (GTK_TABLE (table), entry, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
                    
                    

  label = gtk_label_new ("Initial value :");
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 5, 5);
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);

  entry = gtk_entry_new ();
  wpelem->value = GTK_ENTRY(entry);
  gtk_widget_show (entry);
  gtk_table_attach (GTK_TABLE (table), entry, 1, 2, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
                    


  table2 = gtk_table_new (2, 2, FALSE);
  gtk_widget_show (table2);
  gtk_table_attach (GTK_TABLE (table), table2, 1, 2, 3, 4,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);

  checkbutton = gtk_check_button_new_with_label ("Create set function");
  wpelem->setfunc = GTK_TOGGLE_BUTTON(checkbutton);
  gtk_widget_show (checkbutton);
  gtk_table_attach (GTK_TABLE (table2), checkbutton, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  gtk_signal_connect(GTK_OBJECT(checkbutton),"toggled",
                     GTK_SIGNAL_FUNC(on_btnSetFunc_toggled),
                     (gpointer)wpelem);

  checkbutton = gtk_check_button_new_with_label ("Allocate set value");
  wpelem->allocset = GTK_TOGGLE_BUTTON(checkbutton);
  gtk_widget_show (checkbutton);
  gtk_table_attach (GTK_TABLE (table2), checkbutton, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  gtk_signal_connect(GTK_OBJECT(checkbutton),"toggled",
                     GTK_SIGNAL_FUNC(on_btnAllocSet_toggled),
                     (gpointer)wpelem);

  frame = gtk_frame_new("Allocation code");
  gtk_widget_show(frame);
  gtk_table_attach(GTK_TABLE(table2), frame, 1, 2, 0, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);

  scrollwin = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrollwin);
  gtk_container_add(GTK_CONTAINER(frame),scrollwin);

  text = gtk_text_new(NULL,NULL);
  wpelem->setcode = GTK_TEXT(text);
  gtk_widget_show(text);
  gtk_text_set_editable(GTK_TEXT(text),TRUE);
  gtk_text_set_word_wrap(GTK_TEXT(text),FALSE);
  gtk_text_set_line_wrap(GTK_TEXT(text),FALSE);
  gtk_container_add(GTK_CONTAINER(scrollwin),text);

  table2 = gtk_table_new (2, 2, FALSE);
  gtk_widget_show (table2);
  gtk_table_attach (GTK_TABLE (table), table2, 1, 2, 4, 5,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);

  checkbutton = gtk_check_button_new_with_label ("Create get function");
  wpelem->getfunc = GTK_TOGGLE_BUTTON(checkbutton);
  gtk_widget_show (checkbutton);
  gtk_table_attach (GTK_TABLE (table2), checkbutton, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  gtk_signal_connect(GTK_OBJECT(checkbutton),"toggled",
                     GTK_SIGNAL_FUNC(on_btnGetFunc_toggled),
                     (gpointer)wpelem);

  checkbutton = gtk_check_button_new_with_label ("Allocate get value");
  wpelem->allocget = GTK_TOGGLE_BUTTON(checkbutton);
  gtk_widget_show (checkbutton);
  gtk_table_attach (GTK_TABLE (table2), checkbutton, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  gtk_signal_connect(GTK_OBJECT(checkbutton),"toggled",
                     GTK_SIGNAL_FUNC(on_btnAllocGet_toggled),
                     (gpointer)wpelem);

  frame = gtk_frame_new("Allocation code");
  gtk_widget_show(frame);
  gtk_table_attach(GTK_TABLE(table2), frame, 1, 2, 0, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);

  scrollwin = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrollwin);
  gtk_container_add(GTK_CONTAINER(frame),scrollwin);

  text = gtk_text_new(NULL,NULL);
  wpelem->getcode = GTK_TEXT(text);
  gtk_widget_show(text);
  gtk_text_set_editable(GTK_TEXT(text),TRUE);
  gtk_text_set_word_wrap(GTK_TEXT(text),FALSE);
  gtk_text_set_line_wrap(GTK_TEXT(text),FALSE);
  gtk_container_add(GTK_CONTAINER(scrollwin),text);

  return table;
}

static GtkWidget*
gwf_widpropelem_buttons (GwfWidPropElem *wpelem)
{
  GtkWidget	*hbox;
  GtkWidget	*button;
  
  hbox = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox);

  button = gtk_button_new_with_label ("Add");
  gtk_widget_show (button);
  gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 5);

  gtk_signal_connect (GTK_OBJECT (button), "clicked",
                      GTK_SIGNAL_FUNC (on_btnElemAdd_clicked),
                      (gpointer)wpelem);
                      
  button = gtk_button_new_with_label ("Update");
  gtk_widget_show (button);
  gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 5);

  gtk_signal_connect (GTK_OBJECT (button), "clicked",
                      GTK_SIGNAL_FUNC (on_btnElemUpdate_clicked),
                      (gpointer)wpelem);
                      
  button = gtk_button_new_with_label ("Delete");
  gtk_widget_show (button);
  gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 5);

  gtk_signal_connect (GTK_OBJECT (button), "clicked",
                      GTK_SIGNAL_FUNC (on_btnElemDelete_clicked),
                      (gpointer)wpelem);
                      
  button = gtk_button_new_with_label ("Clear");
  gtk_widget_show (button);
  gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 5);

  gtk_signal_connect (GTK_OBJECT (button), "clicked",
                      GTK_SIGNAL_FUNC (on_btnElemClear_clicked),
                      (gpointer)wpelem);
  return hbox;
}

