/*  Gtk+ Widget Factory
 *  Copyright (C) 1999-2000  Jeroen Benckhuijsen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifndef GWF_PROJDATA_SIGNAL_H
#define GWF_PROJDATA_SIGNAL_H

#ifdef __cplusplus
extern "C" {
#endif

#include <gtk/gtk.h>
#include <gwf_lib.h>

typedef struct _offersig		offersig;
typedef struct _sigparam		sigparam;

typedef void	(*GwfOfrsigForeachFunc)		(offersig *signal,gpointer user_data);
typedef void	(*GwfSigparamForeachFunc)	(sigparam *param,gpointer user_data);

struct _sigparam
{
   gchar	*name;
   gchar	*type;
   GtkType	gtktype;
};

struct _offersig
{
   gchar	*name;
   gchar	*retval;
   GtkType	rvtype;
   GList	*param;
};

GwfProjData*	gwf_project_signal_new		(GwfProject *project);

void		gwf_project_add_ofrsig		(GwfProject *project,offersig *signal);
void		gwf_project_new_ofrsig		(GwfProject *project,gchar *name);
void		gwf_project_del_ofrsig		(GwfProject *project,gchar *name);
void		gwf_project_ofrsig_set_name	(GwfProject *project,gchar *name,gchar *newname);
void		gwf_project_ofrsig_set_retval	(GwfProject *project,gchar *name,gchar *retval);
void		gwf_project_ofrsig_set_rvtype	(GwfProject *project,gchar *name,GtkType type);
gchar*		gwf_project_ofrsig_get_retval	(GwfProject *project,gchar *name);
GtkType		gwf_project_ofrsig_get_rvtype	(GwfProject *project,gchar *name);
GList*		gwf_project_ofrsig_get_param	(GwfProject *project,gchar *name);
void		gwf_project_ofrsig_foreach	(GwfProject *project,GwfOfrsigForeachFunc func,gpointer user_data);
offersig*	gwf_project_ofrsig_find		(GwfProject *project,gchar *name);
gint		gwf_project_ofrsig_count	(GwfProject *project);

gint		gwf_ofrsig_add_param 		(offersig *signal,sigparam *param);
gint		gwf_ofrsig_new_param 		(offersig *signal,gchar *name);
gint		gwf_ofrsig_del_param 		(offersig *signal,gchar *name);
gint		gwf_ofrsig_param_set_name 	(offersig *signal,gchar *name,gchar *newname);
gint		gwf_ofrsig_param_set_type 	(offersig *signal,gchar *name,gchar *type);
gint		gwf_ofrsig_param_set_gtktype	(offersig *signal,gchar *name,GtkType type);
gchar*		gwf_ofrsig_param_get_type 	(offersig *signal,gchar *name);
GtkType		gwf_ofrsig_param_get_gtktype	(offersig *signal,gchar *name);
void		gwf_ofrsig_param_foreach 	(offersig *signal,GwfSigparamForeachFunc func,gpointer user_data);
void		gwf_ofrsig_free			(offersig *signal);
sigparam*	gwf_ofrsig_param_find		(offersig *signal,gchar *name);

void		gwf_project_ofrsig_add_param		(GwfProject *project,gchar *signame,sigparam *param);
void		gwf_project_ofrsig_new_param		(GwfProject *project,gchar *signame,gchar *name);
void		gwf_project_ofrsig_del_param		(GwfProject *project,gchar *signame,gchar *name);
void		gwf_project_ofrsig_param_set_name	(GwfProject *project,gchar *signame,gchar *name,gchar *newname);
void		gwf_project_ofrsig_param_set_type	(GwfProject *project,gchar *signame,gchar *name,gchar *type);
void		gwf_project_ofrsig_param_set_gtktype	(GwfProject *project,gchar *signame,gchar *name,GtkType type);
gchar*		gwf_project_ofrsig_param_get_type	(GwfProject *project,gchar *signame,gchar *name);
GtkType		gwf_project_ofrsig_param_get_gtktype	(GwfProject *project,gchar *signame,gchar *name);
void		gwf_project_ofrsig_param_foreach	(GwfProject *project,gchar *signame,GwfSigparamForeachFunc func,gpointer user_data);
sigparam*	gwf_project_ofrsig_param_find		(GwfProject *project,gchar *signame,gchar *name);


#ifdef __cplusplus
}
#endif

#endif

